#!/bin/bash

 ######################################################################################
 # Filename: backend.php
 #
 # This file is part of the PowerDNS JAS plugin.
 # Copyright 2003-2016 by Matthijs Schuurman
 #
 # The PowerDNS JAS plugin is free software under the terms of the
 # GNU General Public License version 3 or any later version.
 #
 # For information about the license please refer to:
 # LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/pdns/
 ######################################################################################


#global settings
DYNDNSROOT="/etc/powerdns/dyndns" #dynamic dns key folder
ERRORLOG="${DYNDNSROOT}.log" #log file (/etc/powerdns/dyndns.log)

#database settings, leave at least one empty for no database update
DBHOST="localhost" #database server: usually "localhost"
DBNAME="pdns" #database name
DBUSER="pdns-dyn" #database user: should have SELECT+UPDATE on $DBNAME.pdns_records
DBPASS="" #database password for $DBUSER

#local IP update settings
DEV="eth0" #device to gather IP from, leave empty for no local IP update
KEY="example.com" #dynamic IP key
NOTIFYEMAIL="hostmaster@example.com" #email address for update notifications
SCPLOCATIONS=("user@ns1.example.com:~/.dyndns.example.com" "user@ns2.example.com:~/.dyndns.example.com") #locations to scp updated IP file to


function checkIPUpdate()
{
 local _KEYFILE="$DYNDNSROOT/$KEY"
 local _OLDIP=""
 local _NEWIP=""

 if [ -f "$_KEYFILE" ]; then
  _OLDIP=`cat $_KEYFILE`
 fi

 _NEWIP=`ip addr show dev $DEV|perl -wne 'if(/\s*inet (\d+\.\d+\.\d+\.\d+)\//){print "$1";}'`

 if [ "$_OLDIP" != "$_NEWIP" ]; then #IP changed
 {
  echo "$_NEWIP" > "$_KEYFILE" #update key file

  for l in ${SCPLOCATIONS[@]}; do #scp key file to target locations
   scp "$_KEYFILE" "$l"
  done

  echo "New IP: $_NEWIP" >&2
  echo >&2
 }
 fi

 return 1;
}


function updateDynamicEntries()
{
 local _KEYFILE=$1
 local _KEY=`basename $_KEYFILE`
 local _IP=`cat "$_KEYFILE"`
 local _IDS=()
 local _RES=""

 #escape values
 _KEY=${_KEY//\'/\\\'}
 _IP=${_IP//\'/\\\'}

 _IDS=(`echo "SELECT id FROM pdns_records WHERE dynamic='$_KEY' AND content<>'$_IP'" \
  | mysql --user="${DBUSER}" --password="${DBPASS}" --host="${DBHOST}" "${DBNAME}"`) #get record IDs that need updating

 if [ $? != 0 ]; then
  echo >&2
  return 0;
 fi

 if [ "$_IDS" != "" ]; then #outdated records found
 {
  unset _IDS[0] #remove id column name
  _IDS=${_IDS[@]} #make string
  _IDS=${_IDS// /, } #added commas in between IDs

  _RES=`echo "UPDATE pdns_records SET content='$_IP' WHERE id IN ($_IDS)" \
   | mysql -v -vvv --user="${DBUSER}" --password="${DBPASS}" --host="${DBHOST}" "${DBNAME}"` #update outdated records with new IP

  if [ $? != 0 ]; then
   echo >&2
   return 0;
  fi

  echo "Power DNS records for $_KEY updated successfully" >&2
  echo >&2
 }
 fi

 return 1
}


#redirect errors to log file
exec > /dev/null
exec 2> ${ERRORLOG}

#create dyndns directory
if [ ! -d "$DYNDNSROOT" ]; then
 mkdir --parents "$DYNDNSROOT"
fi


#check if IP changed
if [ "$DEV" ]; then
 checkIPUpdate
fi

#update dynamic entries
if [ "$DBHOST" -a "$DBNAME" -a "$DBUSER" -a "$DBPASS" ]; then
 for k in $DYNDNSROOT/*; do
  updateDynamicEntries $k
 done
fi


#email error log
if [ -s ${ERRORLOG} ]; then
 cat ${ERRORLOG} | mail -s "Dynamic IP updated" ${NOTIFYEMAIL}
fi


#clean up
exec 2> /dev/null
rm ${ERRORLOG}
